# Reexported for consistency.

#' @export
DelayedArray::rowMaxs

#' @export
DelayedArray::rowMins

#' @export
DelayedArray::rowRanges

#' @export
DelayedArray::colMaxs

#' @export
DelayedArray::colMins

#' @export
DelayedArray::colRanges
